@@ -16,13 +16,13 @@
 %                                 June 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -803,26 +803,37 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   /* Copy postscript to temporary file */
   if (SeekBlob(image,PS_Offset,SEEK_SET) != PS_Offset)
     {
-      DestroyImageInfo(clone_info);
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      (void) fclose(ps_file);
+      ThrowException(exception,CorruptImageError,"ImproperImageHeader",
+        image->filename);
+      goto FINISH_UNL;
     }
   count=ReadBlob(image, 2*MaxTextExtent, magick);
   if (count < 1)
     {
-      DestroyImageInfo(clone_info);
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      (void) fclose(ps_file);
+      ThrowException(exception,CorruptImageError,"ImproperImageHeader",
+        image->filename);
+      goto FINISH_UNL;
     }
 
   if (SeekBlob(image,PS_Offset,SEEK_SET) != PS_Offset)
     {
-      DestroyImageInfo(clone_info);
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      (void) fclose(ps_file);
+      ThrowException(exception,CorruptImageError,"ImproperImageHeader",
+        image->filename);
+      goto FINISH_UNL;
     }
   while (PS_Size-- > 0)
   {
     c=ReadBlobByte(image);
     if (c == EOF)
-      break;
+      {
+        (void) fclose(ps_file);
+        ThrowException(exception,CorruptImageError,"ImproperImageHeader",
+          image->filename);
+        goto FINISH_UNL;
+      }
     (void) fputc(c,ps_file);
   }
   (void) fclose(ps_file);
@@ -833,12 +844,13 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
   /*     printf("Detected:%s  \n",magic_info->name); */
   if(exception->severity != UndefinedException) goto FINISH_UNL;
   if(magic_info->name == (char *) NULL) goto FINISH_UNL;
-  if (LocaleCompare(magic_info->name,"WPG") == 0)
-    goto FINISH_UNL;
-
   (void) strncpy(clone_info->magick,magic_info->name,MaxTextExtent-1);
-  if (LocaleCompare(image_info->magick,clone_info->magick) == 0)
-    (void) strcpy(clone_info->magick,"PS");
+  if (LocaleCompare(clone_info->magick,"PFB") != 0)
+    {
+      ThrowException(exception,CorruptImageError,"ImproperImageHeader",
+        image->filename);
+      goto FINISH_UNL;
+    }
 
     /* Read nested image */
   /*FormatString(clone_info->filename,"%s:%s",magic_info->name,postscript_file);*/
@@ -887,7 +899,7 @@ static Image *ExtractPostscript(Image *image,const ImageInfo *image_info,
     } while (p != (Image *) NULL);
   }
 
-  if ((image->rows == 0 || image->columns == 0) && 
+  if ((image->rows == 0 || image->columns == 0) &&
       (image->previous != NULL || image->next != NULL))
   {
     DeleteImageFromList(&image);
@@ -1099,6 +1111,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
   image->columns = 1;
   image->rows = 1;
   image->colors = 0;
+  image->storage_class=DirectClass;
   (void) ResetImagePixels(image,exception);
   bpp=0;
   BitmapHeader2.RotAngle=0;
@@ -1157,7 +1170,7 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               if (WPG_Palette.StartIndex > WPG_Palette.NumOfEntries)
                 ThrowReaderException(CorruptImageError,"InvalidColormapIndex");
               image->colors=WPG_Palette.NumOfEntries;
-              if (!AcquireImageColormap(image,image->colors))
+              if (AcquireImageColormap(image,image->colors) == MagickFalse)
                 goto NoMemory;
               for (i=WPG_Palette.StartIndex;
                    i < (int)WPG_Palette.NumOfEntries; i++)
@@ -1243,10 +1256,28 @@ static Image *ReadWPGImage(const ImageInfo *image_info,
               else
                 {
                   if (bpp < 24)
-                    if ( (image->colors < (one << bpp)) && (bpp != 24) )
-                      image->colormap=(PixelPacket *) ResizeQuantumMemory(
-                        image->colormap,(size_t) (one << bpp),
+                  if ( (image->colors < (one << bpp)) && (bpp != 24) )
+                    {
+                      PixelPacket
+                        *colormap;
+
+                      size_t
+                        colors;
+
+                      colormap=image->colormap;
+                      colors=image->colors;
+                      image->colormap=(PixelPacket *) NULL;
+                      if (AcquireImageColormap(image,one << bpp) == MagickFalse)
+                        {
+                          colormap=(PixelPacket *)
+                            RelinquishMagickMemory(colormap);
+                          goto NoMemory;
+                        }
+                      (void) memcpy(image->colormap,colormap,colors*
                         sizeof(*image->colormap));
+                      colormap=(PixelPacket *)
+                        RelinquishMagickMemory(colormap);
+                    }
                 }
 
               if ((bpp == 1) && (image->colors > 1))
