@@ -18,13 +18,13 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -61,6 +61,7 @@
 #include "magick/memory_.h"
 #include "magick/mime.h"
 #include "magick/module.h"
+#include "magick/monitor-private.h"
 #include "magick/nt-base-private.h"
 #include "magick/nt-feature.h"
 #include "magick/opencl-private.h"
@@ -72,6 +73,7 @@
 #include "magick/semaphore-private.h"
 #include "magick/signature-private.h"
 #include "magick/splay-tree.h"
+#include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/string-private.h"
 #include "magick/thread_.h"
@@ -79,6 +81,14 @@
 #include "magick/token.h"
 #include "magick/utility.h"
 #include "magick/xwindow-private.h"
+#if defined(MAGICKCORE_XML_DELEGATE)
+#  if defined(MAGICKCORE_WINDOWS_SUPPORT)
+#    if !defined(__MINGW32__)
+#      include <win32config.h>
+#    endif
+#  endif
+#  include <libxml/parser.h>
+#endif
 
 /*
   Define declarations.
@@ -414,24 +424,30 @@ MagickExport const MagickInfo *GetMagickInfo(const char *name,
   if (IsMagickTreeInstantiated(exception) == MagickFalse)
     return((const MagickInfo *) NULL);
   magick_info=(const MagickInfo *) NULL;
-#if defined(MAGICKCORE_MODULES_SUPPORT)
   if ((name != (const char *) NULL) && (*name != '\0'))
     {
       LockSemaphoreInfo(magick_semaphore);
       if (LocaleCompare(name,"*") == 0)
+#if defined(MAGICKCORE_BUILD_MODULES)
         (void) OpenModules(exception);
+#else
+        RegisterStaticModules();
+#endif
       else
         {
           magick_info=(const MagickInfo *) GetValueFromSplayTree(magick_list,
             name);
           if (magick_info == (const MagickInfo *) NULL)
+#if defined(MAGICKCORE_BUILD_MODULES)
             (void) OpenModule(name,exception);
+#else
+            (void) RegisterStaticModule(name,exception);
+#endif
         }
       UnlockSemaphoreInfo(magick_semaphore);
     }
-#endif
   if ((name == (const char *) NULL) || (LocaleCompare(name,"*") == 0))
-    magick_info=(const MagickInfo *) GetRootValueFromSplayTree(magick_list);
+    return((const MagickInfo *) GetRootValueFromSplayTree(magick_list));
   if (magick_info == (const MagickInfo *) NULL)
     magick_info=(const MagickInfo *) GetValueFromSplayTree(magick_list,name);
   return(magick_info);
@@ -845,9 +861,6 @@ static MagickBooleanType IsMagickTreeInstantiated(ExceptionInfo *exception)
               "MemoryAllocationFailed");
 #if defined(MAGICKCORE_MODULES_SUPPORT)
           (void) GetModuleInfo((char *) NULL,exception);
-#endif
-#if !defined(MAGICKCORE_BUILD_MODULES)
-          RegisterStaticModules();
 #endif
           magick_list_initialized=MagickTrue;
         }
@@ -1132,6 +1145,9 @@ static SignalHandler *SetMagickSignalHandler(int signal_number,
   action.sa_flags=0;
 #if defined(SA_INTERRUPT)
   action.sa_flags|=SA_INTERRUPT;
+#endif
+#if defined(SA_ONSTACK)
+  action.sa_flags|=SA_ONSTACK;
 #endif
   status=sigaction(signal_number,&action,&previous_action);
   if (status < 0)
@@ -1331,6 +1347,7 @@ MagickExport void MagickCoreGenesis(const char *path,
   (void) XComponentGenesis();
 #endif
   (void) RegistryComponentGenesis();
+  (void) MonitorComponentGenesis();
   instantiate_magickcore=MagickTrue;
   UnlockMagickMutex();
 }
@@ -1362,9 +1379,13 @@ MagickExport void MagickCoreTerminus(void)
       UnlockMagickMutex();
       return;
     }
+  MonitorComponentTerminus();
   RegistryComponentTerminus();
 #if defined(MAGICKCORE_X11_DELEGATE)
   XComponentTerminus();
+#endif
+#if defined(MAGICKCORE_XML_DELEGATE)
+  xmlCleanupParser();
 #endif
   AnnotateComponentTerminus();
   MimeComponentTerminus();
@@ -1386,6 +1407,7 @@ MagickExport void MagickCoreTerminus(void)
   ModuleComponentTerminus();
 #endif
   CoderComponentTerminus();
+  AsynchronousResourceComponentTerminus();
   ResourceComponentTerminus();
   CacheComponentTerminus();
   PolicyComponentTerminus();
@@ -1483,7 +1505,7 @@ MagickExport MagickInfo *SetMagickInfo(const char *name)
   magick_info=(MagickInfo *) AcquireMagickMemory(sizeof(*magick_info));
   if (magick_info == (MagickInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(magick_info,0,sizeof(*magick_info));
+  (void) memset(magick_info,0,sizeof(*magick_info));
   magick_info->name=ConstantString(name);
   magick_info->adjoin=MagickTrue;
   magick_info->blob_support=MagickTrue;
